/*
 * This file is part of family-shop-mgr.
 *
 * family-shop-mgr is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * family-shop-mgr is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with family-shop-mgr.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Author: Unai IRIGOYEN
 * Date: 12/07/2009
 *
 */

#include <QtXml>

#include "ShoppingTreeItem.h"
#include "ShoppingTreeModel.h"

ShoppingTreeModel::ShoppingTreeModel(QString fileName, QObject *parent)
    : QAbstractItemModel(parent),m_fileName(fileName)
{
    connect(this, SIGNAL(documentChanged()), this, SLOT(save()));
    QFile file(m_fileName);
    if (!file.open(QIODevice::ReadOnly))
        return;
    if (!domDocument.setContent(&file)) {
        file.close();
        return;
    }
    file.close();

    rootItemNode = domDocument.firstChildElement("shoppingList");
    rootItem = new ShoppingTreeItem(rootItemNode, 0);
}

/*******************************************************************/
ShoppingTreeModel::~ShoppingTreeModel()
{
    delete rootItem;
}

/*******************************************************************/
int ShoppingTreeModel::columnCount(const QModelIndex &/*parent*/) const
{
    return 4;
}

/*******************************************************************/
QVariant ShoppingTreeModel::data(const QModelIndex &index, int role) const
{
    if (!index.isValid())
        return QVariant();

    if (role != Qt::DisplayRole)
        return QVariant();

    ShoppingTreeItem *item = static_cast<ShoppingTreeItem*>(index.internalPointer());

    QDomNode node = item->node();

    switch (index.column()) {
        case 0:
            return node.firstChildElement("title").text();
        case 1:
            if(node.toElement().nodeName() == "item")
            {
                return node.firstChildElement("quantity").text();
            }
            return QVariant();
        case 2:
            if(node.toElement().nodeName() == "item")
            {
                return node.firstChildElement("store").text();
            }
            return QVariant();
        case 3:
            if(node.toElement().nodeName() == "item")
            {
                return (bool) node.firstChildElement("checked").text().toInt();
            }
            return QVariant();
        default:
            return QVariant();
    }
}

/*******************************************************************/
Qt::ItemFlags ShoppingTreeModel::flags(const QModelIndex &index) const
{
    if (!index.isValid())
        return 0;

    return Qt::ItemIsEnabled | Qt::ItemIsSelectable | Qt::ItemIsEditable;
}

/*******************************************************************/
QVariant ShoppingTreeModel::headerData(int section, Qt::Orientation orientation,
                              int role) const
{
    if (orientation == Qt::Horizontal && role == Qt::DisplayRole) {
        switch (section) {
            case 0:
                return tr("Category/Item name");
            case 1:
                return tr("Quantity");
            case 2:
                return tr("Store");
            case 3:
                return tr("Checked");
            default:
                return QVariant();
        }
    }

    return QVariant();
}

/*******************************************************************/
QModelIndex ShoppingTreeModel::index(int row, int column, const QModelIndex &parent)
            const
{
    if (!hasIndex(row, column, parent))
        return QModelIndex();

    ShoppingTreeItem *parentItem;

    if (!parent.isValid())
        parentItem = rootItem;
    else
        parentItem = static_cast<ShoppingTreeItem*>(parent.internalPointer());

    ShoppingTreeItem *childItem = parentItem->child(row);
    if (childItem)
        return createIndex(row, column, childItem);
    else
        return QModelIndex();
}

/*******************************************************************/
QModelIndex ShoppingTreeModel::parent(const QModelIndex &child) const
{
    if (!child.isValid())
        return QModelIndex();

    ShoppingTreeItem *childItem = static_cast<ShoppingTreeItem*>(child.internalPointer());
    ShoppingTreeItem *parentItem = childItem->parent();

    if (!parentItem || parentItem == rootItem)
        return QModelIndex();

    return createIndex(parentItem->row(), 0, parentItem);
}

/*******************************************************************/
int ShoppingTreeModel::rowCount(const QModelIndex &parent) const
{
    if (parent.column() > 0)
        return 0;

    ShoppingTreeItem *parentItem;

    if (!parent.isValid())
        parentItem = rootItem;
    else
        parentItem = static_cast<ShoppingTreeItem*>(parent.internalPointer());

    int childCount = 0;
    for(QDomElement childElement = parentItem->node().firstChildElement();
        !childElement.isNull(); childElement = childElement.nextSiblingElement())
    {
        if(childElement.nodeName() == "category" || childElement.nodeName() == "item")
            childCount++;
    }
    return childCount;
}

/*******************************************************************/
bool ShoppingTreeModel::insertRows(int row, int count, const QModelIndex &parent, ItemType type)
{
    ShoppingTreeItem *parentItem;

    if (!parent.isValid())
        parentItem = rootItem;
    else
        parentItem = static_cast<ShoppingTreeItem*>(parent.internalPointer());

    for(int i = 0; i < count; i++)
    {
        QDomElement newElement;
        QDomElement titleElement;
        titleElement.setTagName("title");
        if(type == category)
        {
            newElement.setTagName("category");
            titleElement.setNodeValue(tr("new category"));
            newElement.appendChild(titleElement);
            parentItem->node().insertAfter(newElement, parentItem->child(row + i)->node());
        }
        else if(type == item)
        {
            newElement.setTagName("item");
            titleElement.setNodeValue(tr("new item"));
            newElement.appendChild(titleElement);
            QDomElement quantityElement;
            quantityElement.setTagName("quantity");
            newElement.appendChild(quantityElement);
            QDomElement storeElement;
            storeElement.setTagName("store");
            newElement.appendChild(storeElement);
            QDomElement checkedElement;
            checkedElement.setTagName("checked");
            checkedElement.setNodeValue("0");
            newElement.appendChild(checkedElement);
            QDomElement lastModifiedElement;
            lastModifiedElement.setTagName("lastModified");
            lastModifiedElement.setNodeValue(QDateTime::currentDateTime().toString("dd/MM/yyyy-hh:mm:ss"));
            newElement.appendChild(lastModifiedElement);
            parentItem->node().insertAfter(newElement, parentItem->child(row + i)->node());
        }
    }
    emit documentChanged();
}

/*******************************************************************/
ShoppingTreeModel::removeRows(int row, int count, const QModelIndex &parent)
{
    ShoppingTreeItem *parentItem;

    if (!parent.isValid())
        parentItem = rootItem;
    else
        parentItem = static_cast<ShoppingTreeItem*>(parent.internalPointer());

    for(int i = 0; i < count; i++)
    {
        parentItem->node().removeChild(parentItem->child(i)->node());
    }
    emit documentChanged();
}

/*******************************************************************/
ShoppingTreeModel::addCategory(QModelIndex &index)
{
    ShoppingTreeItem *selectedItem;

    if (!parent.isValid())
        selectedItem = rootItem;
    else
        selectedItem = static_cast<ShoppingTreeItem*>(parent.internalPointer());

    insertRows(selectedItem->row(), 1, parent(index), category);
}

/*******************************************************************/
ShoppingTreeModel::addItem(QModelIndex &index)
{
    ShoppingTreeItem *selectedItem;

    if (!parent.isValid())
        selectedItem = rootItem;
    else
        selectedItem = static_cast<ShoppingTreeItem*>(parent.internalPointer());

    insertRows(selectedItem->row(), 1, parent(index), item);
}

/*******************************************************************/
ShoppingTreeModel::removeElement(QModelIndex &index)
{
    ShoppingTreeItem *selectedItem;

    if (!parent.isValid())
        selectedItem = rootItem;
    else
        selectedItem = static_cast<ShoppingTreeItem*>(parent.internalPointer());

    removeRows(selectedItem->row(), 1, parent(index));
}

/*******************************************************************/
ShoppingTreeModel::save()
{
    QFile file(m_fileName);
    file.remove();
    if (!file.open(QIODevice::WriteOnly))
        return;
    if (!file.write(domDocument.toString(4).toUtf8())) {
        file.close();
        return;
    }
    file.close();
}
